##############################################################################
#CMF Options/blark.  Creates a Blark Folder
#Copyright (c) 2001 Jeff Sasmor and Janix Corporation
#Portions (c) Digital Creations
#Released under the ZPL
#No warranties of any kind provided by anyone. Modify at your own risk.
#Please report bugs to jeff@janix.com. 
##############################################################################
#
__doc__ = """ CMF Options Blark Folder

Blark is a weblog for the CMF. It's default UI emulates slash/squish-dot.

Blark is actually a folder that contains a bunch of documents.  
The user interface creates a weblog with items that are discussable
(or not, if Portal Documents do not have discussions enabled via 
portal_types).  

The blark folder owner has access to a properties display that
lets him set various options for that particular blark.
 
"""

from Globals import HTMLFile, DTMLFile, HTML, package_home, InitializeClass
from Products.CMFDefault.Document import Document,addDocument
from Products.CMFCore.PortalFolder import PortalFolder
from Products.CMFCore.CatalogTool import CatalogTool
from Products.CMFCore.utils import modifyPermissionMappings
from Products.CMFCore.MembershipTool import MembershipTool
from Products.CMFCore import utils
from AccessControl import ClassSecurityInfo   
from AccessControl import getSecurityManager
from ComputedAttribute import ComputedAttribute
import DateTime,time,random,string

# Import permission names
from Products.CMFCore import CMFCorePermissions


# Factory type information -- makes Blark objects play nicely
# with the Types Tool (portal_types)
factory_type_information = {'id': 'Blark',
     'content_icon': 'blark_icon.gif',
     'meta_type': 'Blark Weblog',
     'description': ('A blark weblog allows a user to create a personal weblog or a public news log'),
     'product': 'CMFOptions',
     'factory': 'addBlarkFolder',
     'immediate_view': 'BlarkFolder_view',  
     'actions': ({'id': 'new',
                  'name': 'New Article',
                  'action': 'blark_createNewItem',
                  'category': 'object',                 
                  'permissions':  (CMFCorePermissions.AddPortalContent,)}, 
                 {'id': 'rateme',
                  'name': 'Rate Weblog',
                  'action': 'blark_rateme',
                  'category': 'object',
                  'permissions': (CMFCorePermissions.AddPortalContent,)},
                 {'id': 'edit',
                  'name': 'Weblog Properties',
                  'action': 'blark_edit_form',
                  'category': 'object',                  
                  'permissions':  (CMFCorePermissions.ManageProperties,)},  #should be tied to owner or manager
                 {'id': 'manag',  #spelled this way on purpose
                  'name': 'Maintain Weblog',
                  'action': 'blark_manage_form',
                  'category': 'object',                  
                  'permissions':  (CMFCorePermissions.ManageProperties,)},                  


                 ),                     # End Actions
     }


widthSettings = {'4060':('40%','60%'),'5050':('50%','50%'),'6040':('60%','40%'),
                 '7030':('70%','30%'),'7525':('75%','25%'),'8020':('80%','20%'),'9010':('90%','10%')
                }





def addBlarkFolder(self, id, title='', REQUEST=None):
    """
    Create an empty blarkFolder
    """

    #if the user has been locked out, don't do anything.
    if getattr(self,'noBlark','no') <> 'no' :
        raise RuntimeError, 'Locked out: No weblog creation'
        return  #jic


    bl = Blark(id)
    self._setObject(id, bl)
    
    o = self._getOb(id)
    
    #Set up permissions so that blark can be seen by anyone.
    #note: if you think it would be cool to comment this out, be warned
    #that blark WILL NOT work correctly!
    modifyPermissionMappings(o,{'View': {'Anonymous': 1, 'Reviewer': 1}})
    
    
    #now, unless there's an 'allowSelfReviewing==no' attribute available somewhere
    #in the acq chain, set the local role for the member who created this instance
    #to 'reviewer'
    allowRevu = getattr(self,'allowSelfReviewing','yes')=='yes'

   
    if allowRevu:
        from AccessControl.Role import RoleManager 

        user=getSecurityManager().getUser()
        name=user.getUserName()
        if name != 'Anonymous User':
            o.manage_setLocalRoles(name, ['Owner','Reviewer']) 
            
            
    #create an article
    article_id = o.blark_newItem()
    article_obj = o._getOb(article_id)
    
    #Add some text to this article. 
    #BE REALLY CAREFUL if you edit the stuff between the triple-quotes since
    #it is being added as structured-text.  If you add links you must think carefully what happens
    #if your user publishes this article and other portal members can view it. Links to the
    #Weblog Properties page, for example, will give errors to other viewers.  If you use a different
    #subject from Blark in the editMetaData method call, be sure to add that subject to the
    #blark constructor or it won't show up in the subject list.
   
    article_obj.editMetadata(title='Welcome to Blark!', subject='Blark', description='Department of Redundancy')
    article_obj.edit('structured-text',
"""Welcome to Blark
    
   Blark is a weblog for the Zope Content Management Framework. 
   
   Click on the **OPEN** <img src="bfolder_icon.gif" alt="Open"/> icon below to read this article.
   
   You can configure all sorts of customizations via the Weblog Properties
   form link on the left-side menu.
   
   Please read the Help pages!
   
   Basic operation is:
   
   o Click **New Article** to create new articles.
   
   o When viewing the Article list, click on the **Open** icon 
     to view the article, then click **Edit** to add to it or to edit it. 
     You can toggle between View and Edit until you are satisfied with
     how the article looks. You can make the edit page appear
     as soon as you create a new article by toggling an option on the Weblog 
     Properties form.
     
   o When done editing, click Publish or Submit (depending on how your
     system admin has set up Blark operation). Once your article has the
     *Published* status, every logged in Portal member will be able to see it.
     
   o You can delete individual articles by checking the delete checkbox and clicking the
     Trash (garbage-can) icon at the top of this page. The Maintain Weblog form gives you
     a few ways to do mass article deletions.
     
     
   Customizations
   
   o You can change whether or not you're the only one who can post.
   
   o You can set up an access control list and make your weblog totally private for
   you and a few friends.
   
   
   o You can add and change feature-boxes along the right-side of this page
   
   o You can add and change iconic links on the top menu bar to link to URLs or to
     the most recent Article for a particular subject.
     
   o and a lot more: please read the help pages and examine the Weblog Properties
   form.
     
   
   
""")
    
    if REQUEST is not None:
        REQUEST['RESPONSE'].redirect( 'manage_main' )


    


class Blark(PortalFolder):
    """
    A blark is a portal folder that contains Portal Documents.
    """
    meta_type='Blark Weblog'
    isBlarkContainer = 1        #maybe useful later...
    #portal_type = 'Folder'      # ? redundant ?

    # Use Zope 2.3 declarative security
    security = ClassSecurityInfo()
    security.declareObjectProtected(CMFCorePermissions.View)
    security.setPermissionDefault('View',['Anonymous', 'Reviewer'])


   
    def __init__(self, id, title='', description='', text_format='', text='', defaultArticleTitle=None):
        #DefaultDublinCoreImpl.__init__(self,title)
        
        #typical Zopeish and CMFish properties
        self.id=id
        self.title=title
        self.subject=title


        #BE CAREFUL if you tinker with this stuff.  Consider that existing instances may not
        #be compatible if you make changes to some of these attrs.
        #Dict for control info: this way can add new ones later without trashing old instances
        #control info (tbl borders, etc). Only init is to have at least one right-side item
        self.controlInfo = {'rsItems': ['default_rsitem',],'widths':('75%','25%')   }       
        
        #weblog ratings stystem variables
        self._blarkRating = -1                #newly created blark has no rating        
        self.blarkRating = -1                 # a publicly accessible copy
        self._blarkRatingUsers = {}           #dict of users who created rating {user: (rating,date), user:(rating,date),...]
        
        #weblog ACL list
        self._blarkAclList = {}               #dict of users who can access this blark. Empty means everyone. Use dict in case we want to add props to users later.
                                              # key: value = user : {propertyName:value,...}
                                              # currently: user : {access : accessType} where accessType is 'view'  or 'post' (where post implies view as well)

        #the default title for articles
        self.defaultArticleTitle= defaultArticleTitle or 'New Article'
        
        #the subjects list
        self.subjectsList = ['General','Blark']  #blark owner can set a choice of subjects
        
        self._articleInfo = {}            #info about articles. 
                                          # A flexible way to add attrs for each article 
                                          # without having to worry
                                          #about rebuilding folks' blarks for new versions.
                                          # key is article's ID 
                                          # value is: {property:value, property:value, ...}

                                        

    # *********************************************************        
     # Methods having to do with viewing the folder AND some that
     # override methods of Document

    security.declareProtected(CMFCorePermissions.View, 'icon')
    def icon(self):
        """ For the ZMI and Catalog """
        return self.getIcon()

    security.declareProtected(CMFCorePermissions.View, 'folder_contents')
    folder_contents =  HTML('''<dtml-var BlarkFolder_view>''')    #view   

    security.declareProtected(CMFCorePermissions.View, 'index_html')        
    index_html = folder_contents #view


    security.declareProtected(CMFCorePermissions.View, 'blarkMenu')
    blarkMenu = HTML('''<dtml-var blark_menu_horizontal>''')

    security.declareProtected(CMFCorePermissions.View, 'blark_help_home')
    blark_help_home = HTML('''<dtml-var blark_help_main>''')
    
    security.declareProtected(CMFCorePermissions.View, 'metadata_edit_form')
    metadata_edit_form = HTML('''<dtml-var blark_docprops>''')

    security.declareProtected(CMFCorePermissions.View, 'default_rsitem')
    default_rsitem = HTML('''<dtml-var blark_default_rsitem fmt=structured-text>''')

 
    #the next permission is unusual, but needed to allow folks to edit
    #postings in other people's folders (i.e., if they are visiting)
    #even when the Document is pending or private.
    #Otherwise you have to take the folder private to edit a doc - ugh.
    #could add a new role instead...
    #note that the blark_document_edit_form checks access rights so it
    #turns out pretty safe anyway.
    security.declareProtected(CMFCorePermissions.View, 'document_edit_form')
    document_edit_form = HTML('''<dtml-var blark_document_edit_form>''')

    security.declareProtected(CMFCorePermissions.View, '_document_view')
    def _document_view(self):
        """wierdness"""
        return HTML('''<dtml-var blark_document_view>''') 

    
    security.declareProtected(CMFCorePermissions.View, 'document_view')
    document_view = ComputedAttribute(_document_view, 1)

    
    
    
    security.declareProtected(CMFCorePermissions.View, 'content_status_modify')
    content_status_modify = HTML('''<dtml-var blark_content_status_modify>''') 

       
    security.declarePublic('asHTML')
    def asHTML(self): 
        ''' Called when this object is the index_html for a folder.  '''
        view = self.folder_contents       #view
        return apply(view, (self, self.REQUEST))

    security.declareProtected(CMFCorePermissions.View, '__str__')
    def __str__(self):              
        """
        needed to support calling this instance index_html
        """
        return HTML('''<dtml-var BlarkFolder_view>''')   

    # *********************************************************
    #Editing method: handles the properties set in the Weblog Properties form

    security.declareProtected(CMFCorePermissions.ManageProperties, 'edit')
    def edit(self, title=None, description=None,  borders=None, actionsbox=None, 
                   subjectsList=None, globalposting=None, reviewmode=None, 
                   listingSize=None, calendar=None, rsItems=None, 
                   iconicMenuItems=None, disable=None, listingItemSize=None,
                   repliesMode=None,aboutArticle=None,ratingsActive=None,
                   defaultArticleTitle=None,aclUsers=None,listViewSize=None,
                   ownerEmail=None,newItemGoesToEditPage=None,widthMode=None,
                   emailNotify=None,emailNotifyEnable=None,aclForReplies=None,
                   REQUEST=None):
        """                      
        alter various props of the blark folder
        """

        if title is not None: self.title = title
        self.description = description
        
    
        if borders is not None:
            self.controlInfo['borders'] = borders
        else:
            if self.controlInfo.has_key('borders'):
                del self.controlInfo['borders']
                

        if defaultArticleTitle is not None:
            self.defaultArticleTitle = string.strip(defaultArticleTitle)
        else:
            self.defaultArticleTitle = 'New Article'        
              
               
                
        if actionsbox is not None:
            self.controlInfo['actionsbox'] = actionsbox
        else:
            if self.controlInfo.has_key('actionsbox'):
                del self.controlInfo['actionsbox']                



        if emailNotify is not None:
            self.controlInfo['emailNotify'] = self._cleanupLine(emailNotify)
        else:
            if self.controlInfo.has_key('emailNotify'):
                del self.controlInfo['emailNotify']                                     


        if emailNotifyEnable is not None:
            self.controlInfo['emailNotifyEnable'] = emailNotifyEnable
        else:
            if self.controlInfo.has_key('emailNotifyEnable'):
                del self.controlInfo['emailNotifyEnable']        
                
                

        
        if globalposting is not None:
            self.controlInfo['globalposting'] = globalposting
        else:
            if self.controlInfo.has_key('globalposting'):
                del self.controlInfo['globalposting']                
                
        if reviewmode is not None:
            self.controlInfo['reviewmode'] = reviewmode
        else:
            if self.controlInfo.has_key('reviewmode'):
                del self.controlInfo['reviewmode']                
 

        if newItemGoesToEditPage is not None:
            self.controlInfo['newItemGoesToEditPage'] = newItemGoesToEditPage
        else:
            if self.controlInfo.has_key('newItemGoesToEditPage'):
                del self.controlInfo['newItemGoesToEditPage']                
 
        if widthMode is not None:
            self.controlInfo['widthMode'] = widthMode
            self.controlInfo['widths'] = widthSettings.get(widthMode,('75%','25%'))
        else:
            if self.controlInfo.has_key('widthMode'):
                del self.controlInfo['widthMode']                
            if self.controlInfo.has_key('widths'):
                del self.controlInfo['widths']                
 
                
 
                
                
        if listingSize is not None:
            self.controlInfo['listingSize'] = listingSize
        else:
            if self.controlInfo.has_key('listingSize'):
                del self.controlInfo['listingSize']                     
                
                
        if ratingsActive is not None:
            self.controlInfo['ratingsActive'] = ratingsActive
        else:
            if self.controlInfo.has_key('ratingsActive'):
                del self.controlInfo['ratingsActive']                
                
                
        #this feature is currently on hold.
        if calendar is not None:
            self.controlInfo['calendar'] = calendar
        else:
            if self.controlInfo.has_key('calendar'):
                del self.controlInfo['calendar']                     
                
        if disable is not None:
            self.setAnonViewability(0)               
        else:
            self.setAnonViewability(1) #otherwise it is viewable


        if repliesMode is not None:
            self.controlInfo['repliesMode'] = repliesMode
        else:
            if self.controlInfo.has_key('repliesMode'):
                del self.controlInfo['repliesMode']                                     


        if listingItemSize is not None:
            self.controlInfo['listingItemSize'] = listingItemSize
        else:
            if self.controlInfo.has_key('listingItemSize'):
                del self.controlInfo['listingItemSize']                                     
                                
        if listViewSize is not None:
            self.controlInfo['listViewSize'] = listViewSize
        else:
            if self.controlInfo.has_key('listViewSize'):
                del self.controlInfo['listViewSize']                                     
                                
                                
                                
        if ownerEmail is not None:
            self.controlInfo['ownerEmail'] = self._cleanupLine(ownerEmail)
        else:
            if self.controlInfo.has_key('ownerEmail'):
                del self.controlInfo['ownerEmail']                                              
                                
                                
        if aboutArticle is not None:
            self.controlInfo['aboutArticle'] = self._cleanupLine(aboutArticle)
        else:
            if self.controlInfo.has_key('aboutArticle'):
                del self.controlInfo['aboutArticle']                                        
        
                                

        if rsItems is not None:
            self.controlInfo['rsItems'] = self._cleanupLinesField(rsItems,1)
        else:
            if self.controlInfo.has_key('rsItems'):
                del self.controlInfo['rsItems']                     


        if iconicMenuItems is not None:
            self.controlInfo['iconicMenuItems'] = self._cleanupLinesField(iconicMenuItems,1)
        else:
            if self.controlInfo.has_key('iconicMenuItems'):
                del self.controlInfo['iconicMenuItems']                     
               
                
                
        if subjectsList is not None:
            #In this case we make cleanupLinesField remove interior
            #spaces in each line. 
            #this is necc or we'd have to modify Portal Document classe
            #which I wanted to avoid doing. (prob is that subjects with
            #spcs get turned into two words when using the document edit
            #stuff that currently exists.

            testList =  self._cleanupLinesField(subjectsList,1)
            
            #now scan for 'http:' as a subject; that's NG since it interferes with getIconicMenuItems()
            finalList = []
            for item in testList:
                testitem = string.upper(item)
                if string.find(testitem,'HTTP://') <> -1:
                    continue
                finalList.append(item)

            self.subjectsList = finalList
       
       
        if aclUsers is not None:        #aclUsers is a lines property
            aclUsers = self._cleanupLinesField(aclUsers,1)
            #have to transform into {user:{access:mode},...}
            
            al = self._blarkAclList
            alk = al.has_key
            if len(aclUsers)==0: 
                al.clear()

            namesList = []                     
            for line in aclUsers: 

                parts = string.split(line,',',2)
                if len(parts) < 2:                      #skip invalid lines
                    continue                        
                mode = string.lower(parts[1])           #should be view or post, normalize to lowercase
                namesList.append(parts[0])              #make a list of the user names for later use
                
                if mode <> 'view' and mode <> 'post':    #skip invalid line
                    continue
                if alk(parts[0]):                       #see if user is already in list
                    al[parts[0]]['access']=mode         #else modify current access key's value
                else:
                    al[parts[0]] = {}                   #init an empty dict for this user. Maybe this is a silly way to do this...
                    al[parts[0]]['access']=mode         #How does python know this is a dict instead of an indexing operation? hmmm

            #not done yet: compare keys in blarkAckList to users names to see who should be
            #removed from the list.
            
            keys = al.keys()
            for key in keys:                           #scan keys
                if key not in namesList:                #if no match
                    del al[key]                         #then out it goes
             
        else:
            self._blarkAclList.clear()
                        


        if aclForReplies is not None:
            self.controlInfo['aclForReplies'] = aclForReplies
        else:
            if self.controlInfo.has_key('aclForReplies'):
                del self.controlInfo['aclForReplies']                







       
        
        if REQUEST is not None:
            REQUEST['RESPONSE'].redirect(self.absolute_url())



    # *********************************************************
    # Access control functionality
    
    security.declareProtected(CMFCorePermissions.ManageProperties, 'getBlarkAclList')
    def getBlarkAclList(self):
        """ return list of strings: ['user,view'|'post','user,view|post',....]
            Used by UI for display in blark_edit_form
            key  : value
            user : {access : accessType}
        """
        aclList = self._blarkAclList.items()
        
        output = []
        for v in aclList:
            output.append('%s,%s' % (v[0],v[1].get('access','view') ) ) #access mode defaults to view
            
        return output
        
            
    security.declareProtected(CMFCorePermissions.View, 'isUserInBlarkAclList')
    def isUserInBlarkAclList(self,user):
        """ if length of acl list is > 0, check for match. If list len is 0 then we
            aren't using this feature, return 'post' (basically = unrestricted)
            if list len > 0 and user in list, return view or post from the list
            if user not in list, return ''
        """
        
        
        if len(self._blarkAclList) == 0: 
            return 'post'       #unrestricted
        userprops = self._blarkAclList.get(user)  #get the user properties for this user
        if userprops == None:   #not in the list
            allView = self._blarkAclList.get('****') #check for global view setting (user = ****)
            if allView <> None:
                return 'view'
            else:
                return ''
                
        
        return userprops.get('access','') #return the access mode or ''

        



    # *********************************************************
    # Methods called by the Folder's UI 

    security.declareProtected(CMFCorePermissions.View, 'getIconicMenuItems')
    def getIconicMenuItems(self):
        """ scans a list of items ordered id,URL id,URL... and builds a list of
            tuples [(id,URL),(id,URL)....].  The original id,URL list is 
            what's set up by the Blark owner in the Weblog properties page'
            Menu Bar Items form field.
            
            Used to build the horizontal menu on the Blark home page.
        """
        ikons = self.controlInfo.get('iconicMenuItems',None)
        if ikons == None or ikons == []:
            return []

        id_url_pairs = []
        
        base = self.findContainingBlarkFolder().absolute_url()
        
        for ikon in ikons:
            ik = string.split(ikon,',',2)
            if len(ik) < 2: 
                continue

            #check ik[1] for a URL or a subject. 
            urlsubj = ik[1]
            if urlsubj in self.subjectsList:        #valid subject? 
                #yes, create a valid URL
                urlsubj='%s/blarkArtFind?s=%s' % (base,urlsubj) 
            id_url_pairs.append( (ik[0], urlsubj ) ) #tuplize the first 2

        return id_url_pairs



    security.declareProtected(CMFCorePermissions.View, 'blarkArticleFinder')
    def blarkArticleFinder(self,targetSubj,REQUEST):
        """ 
           scans thru articles, looking for the first PUBLISHED article with that subject
           (this would of course be the most recent...).
           Returns the object.  When a user clicks a horizontal menu item that's a subject,
           the link is to a dtml method that calls this method in order to find and 
           display the correct article.
         """

        pm = utils.getToolByName(self,'portal_membership')
        if pm.isAnonymousUser():    #sorry, gotta be logged in.
            return HTML("<dtml-var standard_html_header><h3>Sorry, logged in members only!</h3><dtml-var standard_html_footer>")
        
        content = self.contentValues('Document')
 
        for c in content:
            if targetSubj in c.subject and self.revuState(c)  == 'published':
                #found the most recently posted and published article with that subject.
                return apply(self.document_view, (c, self.REQUEST)) 

        return None



    security.declareProtected(CMFCorePermissions.View,'blarkList')
    def blarkList(self,user,revu=None,showAll=None):
        """  returns a list of filtered Document instances.
             blarkList gets a list of the Documents in the folder & screens out those
            that are not published unless showAll <> None.  The main method
            used to build the 'home page' of the blark (the article list)

        """
        if revu:
            return self.blarkListReviewable(user)
        
        from AccessControl.Role import RoleManager 
        from AccessControl.Owned import Owned
                     
        content = self.contentValues('Document')
        
        
        if showAll:
            return content

        iList = []


        """
        some of this this checking may seem odd - why not just check the 
        review state?
        well, the default workflow won't let you look at it unless
        you are a manager (basically). I wanted to be able to show
        a portal member who is seeing the posting list of ANOTHER portal
        member's Blark to be able to see his postings even if they are
        not 'published'.  This ?hack? avoids changing
        workflows; which was of great importance to me. 
        Presumably any wf will set the Anon role's View permission
        to ON when something is viewable.  This situation is trickier
        since the folder itself is/can be publicly viewable.
        """
        for c in content:
            if user == c.getOwner(1)[1]:  #if the person looking at it is its owner
                iList.append(c)       #then s/he can see it w/ort review state
            else:
                rop=c.rolesOfPermission('View')  #get the roles and perm settings for this obj
                for r in rop:
                    if r.get('name','') == 'Anonymous' :       #for the Anon role
                        if r.get('selected') == 'SELECTED' :   #if it is SELECTED
                            iList.append(c)                    #then you can see it
                        break;

        return iList
        
        
        
    security.declareProtected(CMFCorePermissions.ReviewPortalContent,'blarkListReviewable')
    def blarkListReviewable(self,user):
        """  returns a list of filtered Document instances: only those that are reviewable
             (in the pending state)
             This is invoked instead of blarkList when the user has set the 'Review'
             Mode from the Weblog Properties form
         
        """
                  
        content = self.contentValues('Document')
        
        iList = []

        for c in content:
            if self.blarkErsatzGetReviewState(c,user,1) == 'pending':
                iList.append(c)      #add to the list

        return iList        
        


    security.declareProtected(CMFCorePermissions.View,'blarkListBySubject')
    def blarkListBySubject(self):
        """  Similar to blarkList, but returns a dictionary: 
             {subject:[list of published documents], subject:[list], ...}
             
             The value of each dict item is a list of viewable Document objs 
             for each subject. The keys are the subjects. 
             ONLY returns published Documents, regardless of the user
             
             
             Invoked by blark_ArticlesBySubject
        """
        
        from AccessControl.Role import RoleManager 
                     
        content = self.contentValues('Document')
        
        outDict={}       

        for c in content:
            rop=c.rolesOfPermission('View')  #get the roles and perm settings for this obj
            for r in rop:
                if r.get('name','') == 'Anonymous' :       #for the Anon role
                    if r.get('selected') == 'SELECTED' :   #if it is SELECTED
                        subject = (len(c.subject)>0 and c.subject[0]) or None
                        if subject:
                            if outDict.has_key(subject):
                                outDict[subject].append(c)  #add to the existing list
                            else:
                                outDict[subject] = [c]      #make a new one
                        else:  #handle cases where no one ever specified a subject.
                            if outDict.has_key('Unspecified'):
                                outDict['Unspecified'].append(c)  #add to the existing list
                            else:
                                outDict['Unspecified'] = [c]      #make a new one
                        
                        break; #done with this Document

        return outDict
        
    
    security.declareProtected(CMFCorePermissions.View,'blarkListForSubject')
    def blarkListForSubject(self,subj):
        """  Similar to blarkListBySubject, but returns a list of published
             documents for the passed-in subject. 

             ONLY returns published Documents, regardless of the user
             ignores subjects not in the current subject list
             
             Invoked by blark_ArticlesForSubject
        """
        
        from AccessControl.Role import RoleManager 
                     
        content = self.contentValues('Document')
        
        output = []

        for c in content:
            rop=c.rolesOfPermission('View')  #get the roles and perm settings for this obj
            for r in rop:
                if r.get('name','') == 'Anonymous' :       #for the Anon role
                    if r.get('selected') == 'SELECTED' :   #if it is SELECTED
                        subject = (len(c.subject)>0 and c.subject[0]) or None
                        if subject and subject==subj:
                            output.append(c)  #add to the list
                        break

        return output
    
    
    
    
    security.declareProtected(CMFCorePermissions.View,'blarkListForAuthor')
    def blarkListForAuthor(self,author):
        """  Similar to blarkListBySubject, but returns a list of published
             documents for the passed-in author. 

             ONLY returns published Documents, regardless of the user
             
             Invoked by blark_ArticlesForAuthor
        """
        
        from AccessControl.Role import RoleManager 
                     
        content = self.contentValues('Document')
        
        output = []

        for c in content:
            rop=c.rolesOfPermission('View')  #get the roles and perm settings for this obj
            for r in rop:
                if r.get('name','') == 'Anonymous' :       #for the Anon role
                    if r.get('selected') == 'SELECTED' :   #if it is SELECTED
                        if c.Creator() ==  author:
                            output.append(c)  #add to the list
                        break

        return output
    
    
        
    security.declareProtected(CMFCorePermissions.View,'blarkNumPublished')
    def blarkNumPublished(self):
        """  returns the number of published documents
        """
                 
        content = self.contentValues('Document')
        
        count = 0

        for c in content:
            if self.revuState(c)  == 'published':
                count = count + 1

        return count
        
        
        
    security.declareProtected(CMFCorePermissions.View,'blarkErsatzGetReviewState')
    def blarkErsatzGetReviewState(self,obj,user,showAll=None):
        """  
            blarkErsatzGetReviewState returns review state or nothing if 
            user is not owner of the document.  Returns a value for display
            or for use in additional processing.
 
             returns 'private', 'pending' or 'published' depending on
             permission settings for certain roles.
             
             user is the authenticated user
             showAll is not None if the blark that calls this method is being
             viewed by its owner or a manager or reviewer
             
        """
        
        if (showAll is None) and (user <> obj.getOwner(1)[1]):    
                         #user is not the the blark 'superuser' (its owner or a 
                         #manager or reviewer- as determined by some dtml
                         #upstream of this method)
                         #and is NOT owner of this obj
            return ''    #forget it, buddy!
        
        return self.revuState(obj)
        
        
    security.declareProtected(CMFCorePermissions.View,'revuState')        
    def revuState(self,obj):
        """ examines roles of Permissions to determine
           review state for case where the viewer of a blark is not the owner.
           Makes reasonable assumptions that:
           Anon role's View permission = to ON when something is published
           Reviewer roles's View permission is ON when something is pending or published
           Otherwise it's Private
    
          like blarkList above YMMV if you use a different workflow that does something
          else.
       """

        from AccessControl.Role import RoleManager 
        from AccessControl.Owned import Owned
        
        
        rop=obj.rolesOfPermission('View')  #get the roles and perm settings for this obj
        for r in rop:
            if r.get('name','') == 'Anonymous' :       #for the Anon role
                if r.get('selected') == 'SELECTED' :   #if it is SELECTED
                    return 'published'
            if r.get('name','') == 'Reviewer' :       #for the Anon role
                if r.get('selected') == 'SELECTED' :   #if it is SELECTED
                    return 'pending'
        return 'private'        



    security.declareProtected(CMFCorePermissions.AddPortalContent, 'blark_newItem')
    def blark_newItem(self,REQUEST=None):
        """Add a new document into the folder"""
        
        #make up a random number based id
        num_id = random.random()  * 1000000000
        str_id = str(int(num_id))
        logtime = str(int(time.time() ))
        str_id = 'BLARKENTRY_' + str_id + '_' + logtime   

        #either one of these works
        #self.invokeFactory('Document', str_id, title=self.defaultArticleTitle)
        addDocument(self,str_id,self.defaultArticleTitle)

        ob = self._getOb(str_id, self)
        if ob is self:
            REQUEST['RESPONSE'].redirect('%s?portal_status_message=Error creating article!' % self.absolute_url())


        
        article=getattr(self, str_id)          #perhaps somewhat redundant....
        setattr(article,'view',self.document_view)  #change the view attr to point to our view attr
        #note that article.view = self.document_view doesn't work.
        
        modifyPermissionMappings(article, 
                                 {'View': {'Anonymous': 0, 'Member': 0, 'Reviewer': 0, 'Owner': 1, 'Manager' : 1},  
           'Access contents information': {'Anonymous': 0, 'Member': 0, 'Reviewer': 0, 'Owner': 1, 'Manager' : 1} })     



        
        self._articleInfo[str_id]={'exists':'yes'} #each article has its own dictionary of props
                                              #note that _articleInfo is a dict, each key:value pair is
                                              # article_id : {Property:value, Property:value, ...}

        
        if REQUEST is not None:
            #REQUEST['RESPONSE'].redirect('%s/%s/document_edit_form' % (self.absolute_url(),str_id) )
            REQUEST['RESPONSE'].redirect('%s?portal_status_message=%s Added to top of list.' % (self.absolute_url(),self.defaultArticleTitle))

        return str_id

       
     
    security.declarePublic('findContainingBlarkFolder')
    def findContainingBlarkFolder(self):
        """ 
        Similar in process to the objectManager superValues method, 
        findContainingBlarkFolder() goes up the tree of 
    
            reply->document->blark folder 
       
        as necc. to find the containing blark folder. It stops at
        the first one it finds.
        
        Note that this method cannot be used except inside a blark folder
        as it does no special exception checking.
        """
         
        obj=self
        x=0
        
        while x < 100:
            if obj['meta_type'] == 'Blark Weblog':
                return obj
            if hasattr(obj,'aq_parent'): 
                obj=obj.aq_parent
            else:
                break;
            x = x + 1

        return None
        
        
    security.declareProtected(CMFCorePermissions.View,'replyInfo')                
    def replyInfo(self,obj):
        """ returns the number of replies attached to a Document 
            be sure obj is a Document instance.
        """
        replies = obj.objectItems()
        replyobjs = (replies and replies[0][1].objectItems()) or None
        nreplies =  (replyobjs and len(replyobjs)) or 0
        return nreplies


    # *********************************************************
    #special access control
                   

    security.declareProtected(CMFCorePermissions.View,'checkViewable')        
    def checkViewable(self,obj,perm):
        """ from obj, and role of user determines if viewable document.
            if so, return '' else return error message
            'perm' is 'viewable' if the caller is a viewing method or
            'editable' if the caller is an editing method
        """
        
        from AccessControl.Role import RoleManager 
        from AccessControl.Owned import Owned
        
        debug = 1
        
        pm = utils.getToolByName(self,'portal_membership')
        isManager = pm.checkPermission('Manage Portal',self)
        isReviewer = pm.checkPermission('Review portal content',self)
        
        if isManager or isReviewer: 
            return ''                   #can always look



        #else base access on access type, review state and type of user
        anon = pm.isAnonymousUser()
        uname = pm.getAuthenticatedMember().getUserName()

        userAcl = (not anon and self.isUserInBlarkAclList(uname)) or ''

        status = self.revuState(obj)
        isDocOwner = (not anon and (uname == obj.getOwner(1)[1])) or 0

        bFolder = self.findContainingBlarkFolder()
        isBFOwner = (not anon and (uname == bFolder.getOwner(1)[1])) or 0 #true if this user is weblog owner

       
        #rules for anonymous visitors, or for those who are not 
        #the weblog owner or the article owner
        if anon or ((not isBFOwner) and (not isDocOwner)):
            if perm <> 'viewable':
                return 'Editing access denied to Guest'
            if status <> 'published':
                return 'Access denied to unpublished content'
            if userAcl == '':
                return 'Access denied: private weblog'
            return ''   #it's editable and published: OK
            
        #rules for document owners (can be weblog owner if this is his/her document)
        if isDocOwner:
            if userAcl == '':
                return 'Access denied: private weblog'  #maybe they used to be allowed in...
            if perm == 'viewable':  #owner can always view a document 
                return ''           #no need to test for view acl, if in this test we are either view or post
            if status <> 'private':  #can only edit when status is private
                return 'Please return content to PRIVATE first'
            if userAcl <> 'post':    #can only edit with posting access
                return 'Access denied: private weblog  (you do not have editing access)'
            return ''               #editable and private: OK
        
        #rules for the Blark owner
        if isBFOwner: 
            if perm == 'viewable':  #owner can always view a document
                return ''
            if status == 'published':  #can only edit when status is private or pending
                return 'Please return content to PRIVATE first'
            return ''
            
         #rules for everyone else (shouldn't be anyone else, but...)
                  
            
        if debug == 1:  return 'Access Denied to user:%s itemstat:%s reqperm: %s' % (uname,status,perm)
        else: return 'Access Denied to user %s' % uname
        
       
        

        
        

    security.declareProtected(CMFCorePermissions.ManageProperties,'setAnonViewability')        
    def setAnonViewability(self,on=0):
        """ set anonymous role's view permission for this folder """
        bFolder = self.findContainingBlarkFolder() #ensure we're modifying perms on the right thing
        if bFolder == None:
            raise NameError, 'Weblog folder not found'

        #if the user has been locked out, don't do anything.
        if getattr(self,'noBlark','no') <> 'no' :
            raise RuntimeError, 'Locked out: Cannot enable'
            return  #jic


        if on > 0: on = 1
      
        modifyPermissionMappings(bFolder,{'View': {'Anonymous': on}})        

    security.declareProtected(CMFCorePermissions.View,'getAnonViewability')        
    def getAnonViewability(self):
        """ return true if blark folder is viewable by the public """
        from AccessControl.Role import RoleManager 
        from AccessControl.Owned import Owned
        
        bFolder = self.findContainingBlarkFolder() #ensure we're checking perms on the right thing
        if bFolder == None:
            return 0
                
        rop=bFolder.rolesOfPermission('View')  #get the roles and perm settings for this obj
        for r in rop:
            if r.get('name','') == 'Anonymous' :       #for the Anon role
                if r.get('selected') == 'SELECTED' :   #if it is SELECTED
                    return 1
        return 0

    # *********************************************************
    #weblog and article ratings subsystem
      
    security.declareProtected(CMFCorePermissions.View, 'postBlarkRating')
    def postBlarkRating(self,user=None,rating=None):
        """ posts a new rating for the blark (as a whole)
        """
        if user == 'Guest': return       #bail. This shouldn't happen

        #policy is to ignore user if s/he posted a rating in the last 30 days.
        #note that method that computes ratings throws out oldest ratings if
        #there are more than 100 of them so this interaction can get complex...
        alreadyRatedIt = self._blarkRatingUsers.get(user,None)
        today = DateTime.DateTime()     #get 'now'
        
        #alreadyRatedIt is (rating,date) 
        if alreadyRatedIt:      #this user already rated this blark: see how long ago it was.
            today_days = long(today)    #of days since 1/1/1901
            rating_days = long(alreadyRatedIt[1]) #ditto
            diff = today_days - rating_days
            if diff < 30: return
        
        
        self._blarkRatingUsers[user]=(int(rating),today)
        self._computeBlarkRating()
        
    security.declarePrivate(CMFCorePermissions.View, '_computeBlarkRating')        
    def _computeBlarkRating(self):
        """nothing fancy, just a straight average. Prunes out oldest rating if # ratings > 100
        """
       
        ratings = self._blarkRatingUsers.items()   # [(user,(rating,date)),...]
        rLen = len(ratings)
       
        total = 0
        oldest = None
        for r in ratings:
            total = total + int(r[1][0])
            r_time = r[1][1].millis()  #datetime of this rating in millisecs 
            if oldest == None or r_time < oldest[1] :  #uninitialized, or if item's millis < oldest's millis then it's older
                oldest = (r[0], r_time)         #this is the oldest entry
        
        self._blarkRating = total/rLen
        self.blarkRating = self._blarkRating  #this is publicly accessible, but frequently refreshed.
        
        if rLen > 100:
            del self._blarkRatingUsers[oldest[0]]  #delete the oldest entry if there are > 100 of them
      

        
        
    security.declareProtected(CMFCorePermissions.View, 'getBlarkRating')        
    def getBlarkRating(self):
        """ returns the rating for the blark
        """
  
        rLen = len(self._blarkRatingUsers)
        return (self._blarkRating, rLen)

 




    security.declareProtected(CMFCorePermissions.View, 'postBlarkArticleRating')
    def postBlarkArticleRating(self,articleId=None, user=None,rating=None):
        """
        post a rating for an article
        """
        
        if articleId==None or user == 'Guest': return       #bail. This shouldn't happen
        
        today = DateTime.DateTime()     #get 'now'        
        articleDict = self._articleInfo[articleId]                 #get the properties dict for this article
        """ 
        two things in articleInfo pertain to ratings: 
        ratings dict holds {user:(rating,date), user:(rating,date),...} info
        rating holds the actual rating.
        """

        
        if articleDict.has_key('ratings'):
            ratingsDict = articleDict.get('ratings', None)  #[(user,(rating,date)),...]
   
            #policy is to ignore user if s/he posted a rating in the last 30 days.
            #note that method that computes ratings throws out oldest ratings if
            #there are more than 100 of them so this interaction can get complex...
            alreadyRatedIt = ratingsDict.get(user,None)

        
            #alreadyRatedIt is (rating,date) 
            if alreadyRatedIt:      #this user already rated this blark: see how long ago it was.
                today_days = long(today)    #of days since 1/1/1901
                rating_days = long(alreadyRatedIt[1]) #ditto
                diff = today_days - rating_days
                if diff < 30: 
                    return
            
        else:
            self._articleInfo[articleId]['ratings'] = {}  
            
        self._articleInfo[articleId]['ratings'][user] = (int(rating),today)

        """nothing fancy, just a straight average. Prunes out oldest rating if # ratings > 100
        """
       
        ratingsDict = articleDict.get('ratings', None)  #need a current copy!
        ratings = ratingsDict.items()   # [(user,(rating,date)),...]
        rLen = len(ratings)
       
        total = 0
        oldest = None
        for r in ratings:
            total = total + int(r[1][0])
            r_time = r[1][1].millis()  #datetime of this rating in millisecs 
            if oldest == None or r_time < oldest[1] :  #uninitialized, or if item's millis < oldest's millis then it's older
                oldest = (r[0], r_time)         #this is the oldest entry
        
        self._articleInfo[articleId]['rating'] = total/rLen

        
        if rLen > 100:
            del self._articleInfo[articleId]['ratings'][oldest[0]]  #delete the oldest entry if there are > 100 of them
      

        
        
    security.declareProtected(CMFCorePermissions.View, 'getBlarkArticleRating')        
    def getBlarkArticleRating(self,articleId=None):
        """ returns rating info to forms etc.
        """
        if articleId==None:        #bail. This shouldn't happen
            return (0,0)
        
        
        rdict = self._articleInfo[articleId].get('ratings')
        rating = self._articleInfo[articleId].get('rating')
        
        if rdict==None or rating==None: 
            return (0,0)
        
        rLen = len(rdict)
        return (rating, rLen)


    # *********************************************************       
    # ZMI method overrides
    
    #OVERRIDE the method in PortalFolder.py so we can use the ZMI correctly
    security.declareProtected(CMFCorePermissions.AddPortalContent, 'manage_addBlark')
    def manage_addBlark(self, id, title='', REQUEST=None):
        """Add a new Blark(subclass of PortalFolder) object with id *id*.
        """
        ob=addBlarkFolder(id, title)
        self._setObject(id, ob)
        if REQUEST is not None:
            return self.BlarkFolder_view(
                self, REQUEST, portal_status_message="Blark Folder added. You're so lucky!")


        
           
    security.declareProtected('Delete objects','manage_delObjects')
    def bmanage_delObjects(self, ids=[], REQUEST=None):
        """Delete a subordinate object
        
        The objects specified in 'ids' get deleted.
        
        
        override the version from OFS.ObjectManager so that the ZMI can
        delete objects from the blark folder AND fix up the mappings
        that are maintained in Blark.
       
        """
        if type(ids) is type(''): ids=[ids]
        if not ids: return 'No items were specified!'
                
        try:    p=self._reserved_names
        except: p=()
        
        for n in ids:
            if n in p:
                return '<EM>%s</EM> cannot be deleted.' % n
                       
        while ids:
            id=ids[-1]
            v=self._getOb(id, self)
            if v is self:
                raise 'BadRequest', '%s does not exist' % ids[-1]
            self._delObject(id)
            
            if self._articleInfo.has_key(id):        #and try to delete the article info as well.
                 del self._articleInfo[id]

            del ids[-1]                                 #delete the item, too, BTW

        return ':OK:'



    # ********************************************************
    #helper functions for replies display (skins/blark_showThreads.dtml)
    security.declareProtected(CMFCorePermissions.View, 'blark_replies')
    def blark_replies(self,tbObj, nReplies=16,maxDepth=16):
        """ makes a list of tuples for the replies display:
            [(title,(text,indent_level,Creator,Date,index,url)),...],
           Does not format the display in any way.
           tbObj is the TOP LEVEL discussion container. We recurse down
           to maxDepth (default = 16 deep) levels of replies.
           Note that nReplies limits max # of replies including all subreplies.
           Default for nReplies=16
           return values: most are obvious.
            indent_level is the heirarchy level (reply, reply.reply, reply.reply.reply,...)
            index is just the sequence # as the replies are traversed. Useful for debugging
            or for setting new limits to the reply list.
        """
        from Products.CMFDefault.DiscussionItem import DiscussionItemContainer
        
        if tbObj == None: return []     #just in case passed a null Discussion Container
        
        #process replies
        replies = tbObj.getReplies()    #get list of objects
        count,output = self._repfmt(replies,nReplies,maxDepth)    #this recurses till done
        return output
        
         
    security.declarePrivate('_repfmt')
    def _repfmt(self, replies, nReplies, maxDepth, level=0, count=0):
        """  generates a list of tuples for one Discussion container.
             returns number of items processed
        """
        from Products.CMFDefault.DiscussionItem import DiscussionItem
        
        limit = nReplies - 1
        
        if level > maxDepth or count > limit:
            return 0,[]
        
        output = []
        
        for reply in replies:
            info = (reply.title,(reply.cooked_text,level,reply.creator,reply.bobobase_modification_time(),count,reply.absolute_url()))
            output.append(info)
            count = count + 1
            if count > limit:
                break

             
            #if there is at least one reply to this reply                            
            if reply.hasReplies():   #recursion
                count,o = self._repfmt(reply.getReplies(),nReplies,maxDepth,level+1, count)
                output = output + o
            
            #have we reached the limit?
            if count > limit:
                break

                
        return count,output
            






    # *********************************************************
    # Utilities

    security.declarePrivate('_cleanupLine')
    def _cleanupLine(self,line):
        "removes leading, trailing, interior spaces"
        line = string.strip(line)
        if len(line) > 0:
            goodline=''
            for char in line:
                if char <> ' ':
                     goodline = goodline + char
            return goodline
        return ''


    security.declarePrivate('_cleanupLinesField')
    def _cleanupLinesField(self,inputList,removeSpaces=None):
        """ Cleans up input lines in a 'lines' form box
            by removing leading/trailing spaces and interior
            spaces if 'removeSpaces' <> None
        """
        newList=[]
        for line in inputList:
            line = string.strip(line)
            if len(line) > 0:
                if removeSpaces:
                    #now go thru the line and remove any interior spaces
                    goodline=''
                    for char in line:
                        if char <> ' ':
                            goodline = goodline + char
                    newList.append(goodline)
                else:
                    newList.append(line)
        return newList



        


    security.declarePublic('blark_chop')
    def blark_chop(self, input, size=None, etc=None):
        """ trunc an input string @ size. Like <dtml-var xxx size=nnn> but
            nnn can be a variable. Code borrowed from DT_Var.py, except for
            the stuff that handles extra spaces
        """
        input_len = len(input)
        if (not size) or (input_len == 0):
            return input        
        
        size = int(size)

        if input_len > size:
            input = input[:size]
            if etc: 
                input = input + ' ' + etc
            else:
                input = input + ' ' + '...'

        return input


    security.declareProtected(CMFCorePermissions.View, 'getUserEmail')
    def getUserEmail(self,userId):
        """ returns a user's email address as submitted in their personalization form """
        
        pm = utils.getToolByName(self,'portal_membership')
        user = pm.getMemberById(userId)
        if user == None: return ''
        else: return user.email
        
    security.declarePrivate('_dateCompare')
    def _dateCompare(self,c0,c1):        
        """ comparison function """
        date0 = long(c0.bobobase_modification_time())
        date1 = long(c1.bobobase_modification_time())
        if date0 < date1: return 1
        if date0 > date1: return -1
        return 0
        
        
        
    security.declareProtected(CMFCorePermissions.View, 'blarkListSortedByModDateOldestLast')
    def blarkListSortedByModDateOldestLast(self,len=None, getIds=None):
        """ returns list of documents sorted by age (mod time) """
        content = self.contentValues('Document')
        content.sort(self._dateCompare)
        if len <> None:
            content = content[int(len):]
        if getIds <> None:
            ids=[]
            for item in content:
                ids.append(item.getId())
            return ids
        else:
            return content




    security.declareProtected(CMFCorePermissions.View,'blarkListArticlesSubject')
    def blarkListArticlesSubject(self,subj, getIds=None):
        """  raw list of documents for the passed-in subject. 
        """
                    
        content = self.contentValues('Document')
        
        output = []

        for c in content:
            subject = (len(c.subject)>0 and c.subject[0]) or None
            if subject and subject==subj:
                output.append(c)  #add to the list

        if getIds <> None:
            ids=[]
            for item in output:
                ids.append(item.getId())
            return ids
        else:
            return output


    
    
    
    
    security.declareProtected(CMFCorePermissions.View,'blarkListArticlesAuthor')
    def blarkListArticlesAuthor(self,author, getIds=None):
        """  raw list of articles for passed-in author
        """
        content = self.contentValues('Document')
        
        output = []

        for c in content:
            if c.Creator() ==  author:
                output.append(c)  #add to the list

        if getIds <> None:
            ids=[]
            for item in output:
                ids.append(item.getId())
            return ids
        else:
            return output
        

        return output


    security.declareProtected(CMFCorePermissions.View,'blarkListAuthors')
    def blarkListAuthors(self):
        """  list of authors
        """
        content = self.contentValues('Document')
        
        output = []

        for c in content:
            creator = c.Creator()
            if creator not in output:
                output.append(creator)  #add to the list
        

        return output


    security.declareProtected(CMFCorePermissions.View,'blark_getPrevNext')
    def blark_getPrevNext(self,thisId):
        """  get previous article and next article. Called by blark_document_view.
             output is a two element tuple (prev OBJ, next OBJ), either one could
             be None. (also tacked on to this tuple is the index of the
             article (the order where it's found), for debugging)
             pass in the id of the current article
        """

        bFolder = self.findContainingBlarkFolder() #ensure we're checking the right thing
        if bFolder == None:
            return (None,None,0)
 

        content = bFolder.contentValues('Document')
        content.reverse()
        cLen = len(content)
        if cLen < 2:    #a boundary condition we can handle right away...
            return (None,None,0)

        endLimit = cLen - 1
        marker = None;        

        index = 0
        for c in content:
            if c.getId() == thisId:
                marker = 1
                break
            index = index + 1
        
        
        if not marker:
            return (None,None,0)      #this shouldn't occur....
        
        #handle other boundary conditions correctly...

        if index == 0:              #at start
            prev_index = None
            next_index = index + 1
        elif index == endLimit:   #at end
            prev_index = index - 1
            next_index = None
        else:
            prev_index = index - 1
            next_index = index + 1
            
            
        p = None
        n = None

        top_stop = endLimit + 1                             #want to check the last article for viewability
        #have to scan up starting at next_index to find a Next article that's viewable by this user
        if next_index <> None:      #if we're not at a boundary (end of article list)
            while 1:
                n_candidate = content[next_index]           #possible candidate for next item
                if self.checkViewable(n_candidate,'viewable') == '':  #can the current user view it?
                    n = n_candidate                         #yes, we're done
                    break   
                next_index = next_index + 1                 #nope, keep looking
                if next_index == top_stop:                   #but we may be at the end.
                    break                                   #if so we're done and n stays = None

        if prev_index <> None:      #if we're not at a boundary (start of article list)
            while 1:
                p_candidate = content[prev_index]           #possible candidate for prev item
                if self.checkViewable(p_candidate,'viewable') == '':  #can the current user view it?
                    p = p_candidate                         #yes, we're done
                    break   
                if prev_index == 0:                           #but we may be at the beginning already, can't back up any more
                   break                                   #if so we're done and p stays = None

                prev_index = prev_index - 1                 #nope, keep looking for more

        return (p,n,index)



    security.declarePublic('blarkProductHome')
    def blarkProductHome(self):
        """a nice home for a link to product info"""
        return 'http://www.ptang.com/blark/'



    security.declareProtected(CMFCorePermissions.View,'blarkSThelp')
    def blarkSThelp(self):
        """  Some words to explain structured text
        """
        return ("""

Text Formatting rules

 You probably noticed that the **EDIT** form mentions two formats: Structured-text and HTML.  Structured-text is a simple format
 that allows you to enter text in a specific, easy to use format. The text is rendered in HTML automatically.  
 
 Structured Text is text that uses indentation and a few simple symbols to perform text formatting.  
 
 You can try it with an article, then see the results by switching between **EDIT** and **VIEW** modes using the links on the left-side menu.

Basic Rules

 - Separate paragraphs with blank lines (*very* important!)

 - Indent paragraphs to create headings

 - Use single or double asterisks (*) around words or phrases to create emphasis (*italics* and **bold**)

 - Use - * or o to create bulleted lists

 - Use a number followed by a space to create numbered lists

Examples

 1. Headings: To create a heading, write the text that you want *followed by a blank line*.  Indent the first word of the next paragraph with one space.  

 2. Italics and bold: to create italics like *this* surround a word or phrase with single asterisks.  To create bold text like **this**, surround a word or phrase with double asterisks.

 3. Underlining: surround a word or phrase with _ (underscore) characters, like _this_ .

 4. To make a list: leave a blank line after each list item, precede each one with - * or o.  If you want a numbered list, use a number followed by a period and a space for each line.

 5. To add a link:  surround the link name (the part you want people to see) in double quotes. Then add a colon followed by the http://www.somewhere.com and at least one blank.  
     like this: "ptang.com":http://www.ptang.com/ 

Note: If you leave a blank line at the end of your entry and the next line has one space on it then you might see the last line of
your input text made into a heading.
""")






















        
    #this is a debugging tool: comment out the 
    # return '' line if you need to use this method.
    security.declareProtected(CMFCorePermissions.View, 'rolefinder')
    def rolefinder(self,obj):
 
        from AccessControl.Role import RoleManager 
        from AccessControl.Owned import Owned
         
        return '' 
         
        inherited= obj.ac_inherited_permissions(1)
        permOfMember = obj.permissionsOfRole('Member')
        permOfAnon = obj.permissionsOfRole('Anonymous')
        permOfOwner = obj.permissionsOfRole('Owner')
        rop=obj.rolesOfPermission('View')   
        ownr=obj.getOwner()

        return 'ownr=%s:inherited=%s<br>MemberPermissions=%s<br>AnonPermisssions=%s<br>OwnerPerm=%s<br>ROP4View=%s<br>' % (ownr,inherited,permOfMember,permOfAnon,permOfOwner,rop)

    
    



manage_addBlark = Blark.manage_addBlark
manage_addBlarkForm = HTMLFile( 'blarkAdd', globals() )


# Intialize the Blark class, setting up security.
InitializeClass(Blark)

