##############################################################################
# 
# Zope Public License (ZPL) Version 1.0
# -------------------------------------
# 
# Copyright (c) Digital Creations.  All rights reserved.
# 
# This license has been certified as Open Source(tm).
# 
# Redistribution and use in source and binary forms, with or without
# modification, are permitted provided that the following conditions are
# met:
# 
# 1. Redistributions in source code must retain the above copyright
#    notice, this list of conditions, and the following disclaimer.
# 
# 2. Redistributions in binary form must reproduce the above copyright
#    notice, this list of conditions, and the following disclaimer in
#    the documentation and/or other materials provided with the
#    distribution.
# 
# 3. Digital Creations requests that attribution be given to Zope
#    in any manner possible. Zope includes a "Powered by Zope"
#    button that is installed by default. While it is not a license
#    violation to remove this button, it is requested that the
#    attribution remain. A significant investment has been put
#    into Zope, and this effort will continue if the Zope community
#    continues to grow. This is one way to assure that growth.
# 
# 4. All advertising materials and documentation mentioning
#    features derived from or use of this software must display
#    the following acknowledgement:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    In the event that the product being advertised includes an
#    intact Zope distribution (with copyright and license included)
#    then this clause is waived.
# 
# 5. Names associated with Zope or Digital Creations must not be used to
#    endorse or promote products derived from this software without
#    prior written permission from Digital Creations.
# 
# 6. Modified redistributions of any form whatsoever must retain
#    the following acknowledgment:
# 
#      "This product includes software developed by Digital Creations
#      for use in the Z Object Publishing Environment
#      (http://www.zope.org/)."
# 
#    Intact (re-)distributions of any official Zope release do not
#    require an external acknowledgement.
# 
# 7. Modifications are encouraged but must be packaged separately as
#    patches to official Zope releases.  Distributions that do not
#    clearly separate the patches from the original work must be clearly
#    labeled as unofficial distributions.  Modifications which do not
#    carry the name Zope may be packaged in any form, as long as they
#    conform to all of the clauses above.
# 
# 
# Disclaimer
# 
#   THIS SOFTWARE IS PROVIDED BY DIGITAL CREATIONS ``AS IS'' AND ANY
#   EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE
#   IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
#   PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL DIGITAL CREATIONS OR ITS
#   CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL,
#   SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT
#   LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF
#   USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND
#   ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY,
#   OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT
#   OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF
#   SUCH DAMAGE.
# 
# 
# This software consists of contributions made by Digital Creations and
# many individuals on behalf of Digital Creations.  Specific
# attributions are listed in the accompanying credits file.
# 
##############################################################################
 
 
__doc__ = """CMFExtImage product module.
	CMFExtImage works like CMF Image, but stores the 
	uploaded image externally in a repository-direcory. 
	
	It wraps the Contributed Zope Product ExtImage. (Part of 'ExFile')
	Some changes were required to that product, so it is included herein to avoid
	having to modify any existing installations.
	
	Unlike ExtImage, we do not support the preview function at all.  It did not
	seem to fit in for this application.
	
	Jeff Sasmor, jeff@janix.com  March 2001
	
	JMS/3/15/2001/Janix 2001
	
	Version 1.1 May 1, 2001  Fixed problem when importing JPEGs, didn't import StringIO
	"""

ADD_CONTENT_PERMISSION = 'Add portal content'

import struct
from Globals import HTMLFile, HTML
from Products.CMFCore.PortalContent import PortalContent
from Products.CMFDefault.Discussions import Discussable
import Globals
from Products.CMFDefault.DublinCore import DefaultDublinCoreImpl
from os.path import isfile
from Products.CMFCore import CMFCorePermissions
from Products.CMFCore.WorkflowCore import WorkflowAction, afterCreate
from Products.CMFOptions.ExtFile import ExtImage
from types import IntType, StringType  #needed to use ExtImage properly
import string #ditto
from cStringIO import StringIO            #added on 1 may 2001.

bad =' ()[]{}ݟ'
good='_______AAAAAAaaaaaaCcEEEEeeeeeIIIIiiiiNnOOOOOOooooooSssUUUUuuuuYYyyZz'
TRANSMAP=string.maketrans(bad, good)

NO_PREVIEW = 0
GENERATE = 1
UPLOAD_NORESIZE = 2
UPLOAD_RESIZE = 3



# Factory type information -- makes CMFExtImage objects play nicely
# with the Types Tool (portal_types)
factory_type_information =  {'id': 'CMFExtImage',
     'meta_type': 'Portal External Image',
     'description': ('External Images are Images that are stored in the filesystem '
                     'rather than in the Zope Database.'),
     'product': 'CMFOptions',
     'icon': 'image_icon.gif',
     'factory': 'addCMFExtImage',
     'immediate_view': 'CMFExtImage_edit_form',
     'actions': ({'name': 'View',
                  'action': 'CMFExtImage_view',
                  'permissions': ('View',)},
                 {'name': 'Edit',
                  'action': 'CMFExtImage_edit_form',
                  'permissions': ('Modify portal content',)},
                 {'name': 'Metadata',
                  'action': 'metadata_edit_form',
                  'permissions': ('Modify portal content',)},

                 ),                     # End Actions
     }
    




def addCMFExtImage( self
            , id
            , title=''
            , file=''
            , content_type=''
            , precondition=''
            , subject=()
            , description=''
            , contributors=()
            , effective_date=None
            , expiration_date=None
            , format='text/html'
            , language='en-US'
            , rights=''
            ):
    """
        Add an Image
    """


    if len(id) > 0:
        id = string.translate(id, TRANSMAP)  #THIS IS A GOOD THING: REMOVE SPACES AND 'CRAP' FROM IDS!!
        self=self.this()
        tempExtImage=CMFExtImage( id, title, file, content_type, precondition, subject
                , description, contributors, effective_date, expiration_date
                , format, language, rights
               )
        self._setObject(id, tempExtImage)
        afterCreate(self._getOb(id))
    
    


class CMFExtImage( ExtImage.ExtImage
           , PortalContent
           , DefaultDublinCoreImpl
           ):
    """
        A Portal-managed Image that is stored in the FileSystem
    """

    # The order of base classes is very significant in this case.
    # Image.Image does not store it's id in it's 'id' attribute.
    # Rather, it has an 'id' method which returns the contents of the
    # instance's __name__ attribute.  Inheriting in the other order
    # obscures this method, resulting in much pulling of hair and
    # gnashing of teeth and fraying of nerves.  Don't do it.
    #
    # Really.
    
    meta_type='Portal External Image'
    effective_date = expiration_date = None
    _isDiscussable = 1
    icon = PortalContent.icon

    __ac_permissions__ = (
        (CMFCorePermissions.ModifyPortalContent, ('edit',)),
        )
    
    def __init__( self
                , id
                , title=''
                , file=''
                , content_type=''
                , precondition=''
                , subject=()
                , description=''
                , contributors=()
                , effective_date=None
                , expiration_date=None
                , format='text/html'
                , language='en-US'
                , rights=''
                
                ):
                    
                              
        ExtImage.ExtImage.__init__(self, id, title, description, 0)
        DefaultDublinCoreImpl.__init__( self, title, subject, description
                               , contributors, effective_date, expiration_date
                               , format, language, rights )
    
#    view = HTMLFile('CMFExtImage_view', globals())			# Public HTML Interface
    
    def __str__(self):   #override the one in ExtImage.py
        return self.tag()
    
    def SearchableText(self):
        """
            SeachableText is used for full text seraches of a portal.
            It should return a concatanation of all useful text.
        """
        return "%s %s" % (self.title, self.description)

    def manage_afterClone(self, item):
        ExtImage.ExtImage.manage_afterClone(self,item)

    def manage_afterAdd(self, item, container):
        """Both of my parents have an afterAdd method"""
        ExtImage.ExtImage.manage_afterAdd(self,item,container)
        PortalContent.manage_afterAdd(self, item, container)

    def manage_beforeDelete(self, item, container):
        """Both of my parents have a beforeDelete method"""
        PortalContent.manage_beforeDelete(self, item, container)
        ExtImage.ExtImage.manage_afterAdd(self,item,container)
        

    def edit(self, file=''):
        """
            Update image.
        """
        if file.filename != '' and file.read() != '':
            self.manage_file_upload(file)

        self.setFormat(self.content_type)

    edit = WorkflowAction(edit)


#ordinarily a string.
    def Format(self):
        return self.format

#added for ExtImage compatibility
    def getFormat(self):
        fmt = self.format
        if string.find(string.lower(fmt),'gif')>0:
            return 'GIF'
        elif string.find(string.lower(fmt),'jpeg')>0:
            return 'JPEG'
        elif string.find(string.lower(fmt),'png')>0:
            return 'PNG'
        else:
            return ''

#added to allow setting of width and height when the ExtImage is loaded to the repository
#liberated (and slightly morphed) from OFS/Image.py
    def getImageInfo(self,filename):
        
        #this handles fixing up the renaming of a file after undo - the
        #first time it's viewed it will be renamed as this method is
        #called (due to call of 'tag' method of the image)
        if not isfile(filename):
            CMFExtImage.inheritedAttribute("_undo")(self)
        
        fp = open(filename,'rb')
  
        data = fp.read(128)

        data = str(data)
        size = len(data)
        height = 0
        width = 0
        content_type = ''           #we don't need this, don't return it, but keep this in case desired later

        # handle GIFs   
        if (size >= 10) and data[:6] in ('GIF87a', 'GIF89a'):
            # Check to see if content_type is correct
            content_type = 'image/gif'
            w, h = struct.unpack("<HH", data[6:10])
            width = int(w)
            height = int(h)
    
        # See PNG v1.2 spec (http://www.cdrom.com/pub/png/spec/)
        # Bytes 0-7 are below, 4-byte chunk length, then 'IHDR'
        # and finally the 4-byte width, height
        elif ((size >= 24) and (data[:8] == '\211PNG\r\n\032\n')
              and (data[12:16] == 'IHDR')):
            content_type = 'image/png'
            w, h = struct.unpack(">LL", data[16:24])
            width = int(w)
            height = int(h)
                
        # Maybe this is for an older PNG version.
        elif (size >= 16) and (data[:8] == '\211PNG\r\n\032\n'):
            # Check to see if we have the right content type
            content_type = 'image/png'
            w, h = struct.unpack(">LL", data[8:16])
            width = int(w)
            height = int(h)
    
        # handle JPEGs
        elif (size >= 2) and (data[:2] == '\377\330'):
            content_type = 'image/jpeg'
            jpeg = StringIO(data)
            jpeg.read(2)
            b = jpeg.read(1)
            try:
                while (b and ord(b) != 0xDA):
                    while (ord(b) != 0xFF): b = jpeg.read(1)
                    while (ord(b) == 0xFF): b = jpeg.read(1)
                    if (ord(b) >= 0xC0 and ord(b) <= 0xC3):
                        jpeg.read(3)
                        h, w = struct.unpack(">HH", jpeg.read(4))
                        break
                    else:
                        jpeg.read(int(struct.unpack(">H", jpeg.read(2))[0])-2)
                    b = jpeg.read(1)
                width = int(w)
                height = int(h)
            except: pass
    
        return width, height

        

    def index_html(self, REQUEST, RESPONSE):
        """
        Display the image
        
        """
         
        return ExtImage.ExtImage.index_html(self,REQUEST,RESPONSE)  

Globals.default__class_init__(CMFExtImage)


# PortalContent classes need to be registered.  This installs the
# class's factory method and icon.
from Products.CMFCore.register import registerPortalContent
registerPortalContent(CMFExtImage,                    # The class to be registered
                      constructors=(addCMFExtImage,), # It's factory (constructor)
                      action='Wizards/Image',   # The URL of it's add interface
                      icon="extImage.gif",         # Filename to take icon from
                      productGlobals=globals())
