"""
blarkUtils.py

*****************************************************************************************
Copyright (c) 2001 Jeff Sasmor and Janix Corp. 
Version 1.0 / 4/19/01  / jms
Released under the ZPL

INSTALL in Zope/Extensions, check permissions and ownership.
Add an External Method in your root folder

not tested with virtual hosting.

******************************************************************************************
findBlarks external method searches for blarks, and returns a list of
tuples [(owner,(title,absolute_url,num_published_postings,viewable,description,rating)),] that 
would be useful in building a page that displays info about the blarks 
within an entire Zope heirarchy.


It will search down the Zope folder heirarchy from wherever you start it, so if
you invoke it from ROOT FOLDER/index_html (for example) it will find all blarks on
that Zope instance.

Its logic assumes that all portals are placed in the folder in which you
invoke this method. Hence, if you invoke this method from a method in the
Root Folder, but your portals are all in a subfolder, you'd need to
bracket the example below with <dtml-with subfolder>

To avoid complex recursions (and to save execution time) this code
does not look for blarks within subfolders.


***Input Parameters***

The parameter 'getCount' controls whether or not the number of
published articles is returned. That count is expensive (timewise) and
will slow your site down if there are a lot of blarks. All the other info
returned from each blark is static information. 

getcount = 0 makes num_published_postings = 0
getcount <> 0 returns the actual value

The parameter 'inside' controls what the method does if it doesn't
find a portal in the current folder. 

inside = 0 makes the method return an empty list if it doesn't find a portal
inside = 1 makes the method assume it's already in a portal and it looks for 
           a member folder with blarks in it. if it doesn't find one then it 
           returns an empty list. Use inside = 1 if you want to make a 
           list of blarks on your portal home page.
           
***Output***           

The output is a list of tuples k. Each tuple has two elements:

k[0] = the owner of the blark. This is the portal login of the creator of the Blark
k[1] = a tuple t of information about the blark

t[0] = the title of the blark.  This may be '' if the owner never titled it.
t[1] = the absolute (fully-qualified) URL of the blark
t[2] = the number of published postings (if getcount was <> 0)
t[3] = 1 if the blark is publicly viewable or 0 if it is not.  Note that
       this will be 0 if the blark owner (or Zope admin) has disabled the blark, 
       so you might want to skip this blark completely (when showing a list) 
       rather than displaying 'Public=no' (as in the example below)
       Note that even if a Blark is public, it may still be access controlled.
       if it is access controlled and you 'go to' it then you'll have an
       opportunity to ask the owner of the log to have access to it.
t[4] = the description field of the blark.
t[5] = the rating of the blark

All of this info is useful when building a list of blarks on a Zope site
or within a portal.  Remember that if you use dtml-in, that sequence-key will
give you k[0] and sequence-item will give you t[].  See the example.

You can try out findBlarks() with the line:

<dtml-var "findBlarks(1,0)">
           
__________________________________________________________________
------------------------ DTML EXAMPLE ----------------------------
__________________________________________________________________


<dtml-let results="findBlarks(1,0)">
  <dtml-if results>
    <table border="1">
     <tr><th>From</th><th>Weblog</th><th>Description</th><th># postings</th><th>Public?</th><th>Rating</th></tr>
     <dtml-in results sort>
       <dtml-let info="_['sequence-item']">
         <tr>
         <td><dtml-var sequence-key></td>
         <td><a href="<dtml-var "info[1]">"><dtml-var "info[0] or 'Blark'"></a></td>
         <td><dtml-var "info[4]" missing>&nbsp</td>
         <td><dtml-var "info[2]"></td>
         <td><dtml-if "info[3]">Yes<dtml-else>No</dtml-if></td>
         <td><dtml-var "info[5]"></td>
         </tr>
       </dtml-let>
     </dtml-in>
     </table>
   <dtml-else>
  What? We have no Blarks today!
  </dtml-if>
</dtml-let>



You can modify this dtml to give you a nice display of weblogs on your Zope site,
or put it in your portal's index_html (found in skins/generic) (be sure to set
inside = 1) for a display of blarks in your portal.

Obviously you need to create an external method through the Zope ZMI before any of
this will work: placing the external method in the root folder is a good idea. 
The examples shown assume it's called findBlarks.

         
         

*******************************************************************************************

"""

def findBlarks(self,getCount,inside):
    
    if inside == 0:
        portals = self.objectValues('CMF Site')  #get list of portals 
        if (portals == None) or (portals == []): #from the dept of redundancy dept
            return []
    else:
        portals = [self,]
    
    blarkList = []

    for portal in portals:
        memberFolders = portal.Members.objectValues('Portal Folder')  #list of member folders
        for folder in memberFolders:
            blarks = folder.objectValues('Blark Weblog')    #blarks in the member folder
         
            for blark in blarks:
                if getCount <> 0:
                    count = blark.blarkNumPublished()
                else:
                    count = 0
                viewable=blark.getAnonViewability()
                r = blark.blarkRating
                r = (r>0 and r) or '?'  #if folder rating == -1 then there's no rating.
                blarkList.append( (blark.getOwner(1)[1],(blark.title,blark.absolute_url(),count,viewable,blark.description,r)) )
    return blarkList
   
   
    